import { ADMIN_FORMSERVICE } from '@admin-client/shared';
import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, existsAsHtmlElement, mock, Mock, MockEvent, mockGetValue } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { TextEditorComponent } from '@ods/component';
import { ButtonComponent, DeleteIconComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { StatistikFieldsFormService } from '../statistik-fields.formservice';
import { AdminStatistikFieldsFormMappingComponent } from './statistik-fields-form-mapping.component';

describe('AdminStatistikFieldsFormMappingComponent', () => {
  let component: AdminStatistikFieldsFormMappingComponent;
  let fixture: ComponentFixture<AdminStatistikFieldsFormMappingComponent>;

  const mappingField: string = getDataTestIdOf('mapping-field-0');
  const removeMappingButton: string = getDataTestIdOf('remove-mapping-0');

  const formBuilder: FormBuilder = new FormBuilder();

  let formService: Mock<StatistikFieldsFormService>;

  beforeEach(async () => {
    const form: FormGroup = formBuilder.group({
      [StatistikFieldsFormService.FIELD_MAPPINGS]: formBuilder.array([
        new FormGroup({ sourcePath: new FormControl(EMPTY_STRING) }),
      ]),
    });

    formService = <any>{
      ...mock(StatistikFieldsFormService),
      form,
      addMapping: jest.fn(),
      removeMapping: jest.fn(),
    };

    mockGetValue(
      formService,
      StatistikFieldsFormService.FIELD_MAPPINGS,
      form.controls[StatistikFieldsFormService.FIELD_MAPPINGS],
    );

    await TestBed.configureTestingModule({
      declarations: [
        AdminStatistikFieldsFormMappingComponent,
        MockComponent(TextEditorComponent),
        MockComponent(ButtonComponent),
        MockComponent(DeleteIconComponent),
      ],
      imports: [ReactiveFormsModule],
      providers: [
        {
          provide: StatistikFieldsFormService,
          useValue: formService,
        },
        {
          provide: ADMIN_FORMSERVICE,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(AdminStatistikFieldsFormMappingComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('mapping input', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, mappingField);
      });
    });

    describe('remove mapping button', () => {
      it('should call formservice', () => {
        dispatchEventFromFixture(fixture, removeMappingButton, MockEvent.CLICK);

        expect(formService.removeMapping).toHaveBeenCalledWith(0);
      });
    });
  });
});
